# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
import sys
import json
import datetime
import time
import base64
from django.utils import timezone
from passlib.hash import md5_crypt

from django.conf import settings
from django.db import models
from django.db.models import Q
from django.contrib.auth.models import Permission, Group
from django.contrib.auth.models import UserManager, AbstractUser
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext
from django_redis import get_redis_connection
from lib.formats import dict_compatibility
from lib.licence import licence_validate, licence_validsms
from lib.security import is_security_mode, is_security_inited
from lib.tools import get_client_ip, trans_time2sec, get_ip_info
from app.utils.ordered_model import OrderedModel
from app.core import constants
from app.utils.fields import ZeroDateField, CharBooleanField, RevCharBooleanField
from django.contrib.auth.models import _user_get_all_permissions, _user_has_perm, _user_has_module_perms
from captcha.models import CaptchaStore


def is_auto_create_dept_maillist():
    from ConfigParser import ConfigParser
    cfg = ConfigParser()
    cfg.read('/usr/local/u-mail/config/custom.conf')
    if cfg.has_option('system', 'auto_create_dept_maillist'):
        flag = cfg.get('system', 'auto_create_dept_maillist')
        return str(flag).strip() == "1"
    return True


class Domain(models.Model):
    """
    域名信息
    """
    id = models.AutoField(primary_key=True, db_column='domain_id')
    domain = models.CharField(_(u'域名'), max_length=50, null=False, blank=False)
    antivirus = models.CharField(_(u'反病毒开关'), max_length=1, choices=constants.FUNCTION_STATUS, default='-1', null=False,
                                 blank=False)
    antispam = models.CharField(_(u'反垃圾开关'), max_length=1, choices=constants.FUNCTION_STATUS, default='-1', null=False,
                                blank=False)
    userbwlist = models.CharField(_(u'黑白名单开关'), max_length=1, choices=constants.FUNCTION_STATUS, default='1',
                                  null=False,
                                  blank=False)
    sendlimit = models.CharField(_(u'发信频率开关'), max_length=1, choices=constants.FUNCTION_STATUS, default='-1',
                                 null=False,
                                 blank=False)
    disabled = models.CharField(_(u'是否禁用'), max_length=1, choices=constants.DISABLED_STATUS, default='-1', null=False,
                                blank=False)

    dkim = models.CharField(_(u'开启DKIM'), max_length=2, default='-1')
    recvsms = models.CharField(_(u'开启发件短信'), max_length=2, default='-1')
    sendsms = models.CharField(_(u'开启收件短信'), max_length=2, default='-1')
    popmail = models.CharField(_(u'开启POP'), max_length=2, default='1')
    spaceclean = models.CharField(_(u'开启空间清理'), max_length=2, default='1')
    impush = models.CharField(_(u'开启IM通信'), max_length=2, default='-1')
    share_domains = models.CharField(max_length=200, default='0', verbose_name=_(u"域名组合情况"))
    share_title = models.CharField(max_length=200, default='0', verbose_name=_(u"所有域企业通讯录组合显示别名"))
    is_wx_host = models.IntegerField(_(u'是否微信主域名'), default='-1')

    class Meta:
        db_table = 'core_domain'
        managed = False
        verbose_name = _(u'域名表')

    def __unicode__(self):
        return self.domain

    @property
    def share_count(self):
        return self.master_domain.count()


class OabShare(models.Model):
    domain = models.ForeignKey(Domain, related_name="master_domain", verbose_name=_(u"域名"))
    view_target = models.ForeignKey(Domain, related_name="refer_domain", verbose_name=_(u"关联共享域名"))
    is_default = models.BooleanField(_(u"默认显示"), default=False)
    created = models.DateTimeField(auto_now_add=True, verbose_name=_(u"创建时间"))

    class Meta:
        managed = False
        db_table = 'oab_share'
        verbose_name = _(u'企业通讯录共享域')
        unique_together = (
            ('domain', 'view_target'),
        )


class CoCompany(models.Model):
    domain_id = models.IntegerField(unique=True)
    company = models.CharField(max_length=100, blank=True, null=True)
    homepage = models.CharField(max_length=100, blank=True, null=True)
    addr_country = models.CharField(max_length=50, blank=True, null=True)
    addr_state = models.CharField(max_length=50, blank=True, null=True)
    addr_city = models.CharField(max_length=50, blank=True, null=True)
    addr_address = models.CharField(max_length=100, blank=True, null=True)
    addr_zip = models.CharField(max_length=20, blank=True, null=True)
    telphone = models.CharField(max_length=20, blank=True, null=True)
    fax = models.CharField(max_length=20, blank=True, null=True)

    class Meta:
        managed = False
        db_table = 'co_company'
        verbose_name = _(u'公司信息表')

    def __unicode__(self):
        return self.company


class Department(OrderedModel):
    """
    部门信息
    """
    domain = models.ForeignKey(Domain, verbose_name=_(u'域名'), null=False, blank=False)
    parent_id = models.IntegerField(_(u'父ID'), default=-1)
    title = models.CharField(_(u'部门名称'), max_length=100, null=False, blank=False)
    order = models.PositiveIntegerField(_(u"显示顺序"), editable=True, db_index=True, db_column="showorder", default=0)
    modlimit = models.TextField(_(u'部门权限'), blank=True, null=True,
                                default='a:2:{s:7:"netdisk";s:1:"1";s:3:"oab";s:1:"1";}')

    class Meta:
        db_table = 'co_department'
        managed = False
        verbose_name = _(u'部门表')

    def __unicode__(self):
        return self.title

    @property
    def next_id(self):
        return self.parent_id if self.parent_id > 0 else 0

    @property
    def level(self, level=0):
        return Department.loopDepartLevel(self, level)

    @staticmethod
    def loopDepartLevel(obj, level):
        if not obj:
            return level
        if obj.parent_id <= 0 or not obj.parent_id:
            return level
        level += 1
        next_obj = Department.objects.filter(id=obj.parent_id).first()
        return Department.loopDepartLevel(next_obj, level)

    @property
    def parent_name(self):
        o = self.parent
        return o and o.get_title or None

    @property
    def has_child(self):
        if Department.objects.filter(parent_id=self.id).exists():
            return True
        return False

    @property
    def parent(self):
        if self.parent_id not in (0, -1):
            return Department.objects.filter(id=self.parent_id).first()
        return None

    @property
    def child_list(self):
        def get_child(d, obj):
            if obj.id in d:
                return d
            d[obj.id] = obj
            for obj in Department.objects.filter(parent_id=obj.id).all():
                get_child(d, obj)
            return d

        # end def
        data = {}
        return get_child(data, self)

    @property
    def get_title(self):
        return ugettext(unicode(self.title))

    @property
    def parent_tree(self):
        """
        获取父部门级数，最高5级
        """
        d = self
        lst = [d.title]
        for i in xrange(5):
            d = d.parent
            if not d:
                break
            lst.append(d.title)
        lst.reverse()
        return u"-->".join(lst)

    def create_mail_list(self):
        if not is_auto_create_dept_maillist():
            return
        from app.maillist.models import ExtList

        obj = ExtList.objects.filter(listtype=u'dept', domain_id=self.domain_id, dept_id=self.id).first()
        if obj:
            obj.listname = self.title
            obj.description = self.title
            obj.save()
            return obj.id
        domain = Domain.objects.filter(id=self.domain_id).first()
        if not domain:
            return 0
        # LINUX-1561 如果有同名的部门 列表名称前面自动加上父部门信息
        dup_lists = ExtList.objects.filter(listname=self.title, listtype='dept')
        title = self.title
        if dup_lists:
            title = self.parent_tree
            for l in dup_lists:
                try:
                    l.listname = Department.objects.get(id=l.dept_id).parent_tree
                    l.save()
                except:
                    pass
        address = u"dept_%s@%s" % (self.id, domain.domain)
        obj = ExtList.objects.create(
            address=address,
            listtype=u'dept',
            domain_id=self.domain_id,
            permission=u'domain',
            dept_id=self.id,
            listname=title,
            description=title
        )
        return obj.id


class CoDepartmentInfo(models.Model):
    # dept_id = models.IntegerField(unique=True)
    id = models.IntegerField(primary_key=True, db_column='dept_id', verbose_name=_(u"部门ID"))
    domain_id = models.IntegerField(_(u'域名'), default=0)
    manager = models.CharField(_(u'部门领导'), max_length=50, blank=True, null=True)
    contact = models.CharField(_(u'联系人'), max_length=50, blank=True, null=True)
    telphone = models.CharField(_(u'电话'), max_length=20, blank=True, null=True)
    fax = models.CharField(_(u'传真'), max_length=20, blank=True, null=True)
    email = models.CharField(_(u'E-Mail'), max_length=80, blank=True, null=True)
    address = models.CharField(_(u'地址'), max_length=100, blank=True, null=True)

    class Meta:
        managed = False
        db_table = 'co_department_info'
        verbose_name = _(u'部门信息表')


class Mailbox(AbstractUser):
    """
    邮件账号信息
    """
    id = models.AutoField(primary_key=True, db_column='mailbox_id')
    domain = models.ForeignKey(Domain)
    domain_str = models.CharField(_(u'域名'), max_length=50, null=False, blank=False, db_column='domain')
    name = models.CharField(_(u'邮箱名称：'), max_length=80, null=False, blank=False)
    quota_mailbox = models.IntegerField(_(u'邮箱容量：'), default=100)
    quota_netdisk = models.IntegerField(_(u'网络硬盘容量：'), default=100)
    limit_send = models.CharField(_(u'发信权限：'), max_length=2, choices=constants.MAILBOX_SEND_PERMIT, default='-1',
                                  null=True, blank=True)
    limit_recv = models.CharField(_(u'收信权限：'), max_length=2, choices=constants.MAILBOX_RECV_PERMIT, default='-1',
                                  null=True, blank=True)
    use_group = models.IntegerField(_(u'是否应用组权限'), default=1)
    # limit_pop = RevCharBooleanField(u'POP功能：', default=True)
    # limit_imap = RevCharBooleanField(u'IMAP功能：', default=True)
    # disabled = RevCharBooleanField(u'邮箱账号状态：', default=True)
    # 与组权限冲突
    # limit_pop = models.CharField(u'POP功能', max_length=2, default='-1', choices=constants.MAILBOX_DISABLED)
    # 与组权限冲突
    # limit_imap = models.CharField(u'IMAP功能', max_length=2, default='-1', choices=constants.MAILBOX_DISABLED)
    disabled = models.CharField(_(u'邮箱账号状态'), max_length=2, default='-1', choices=constants.MAILBOX_DISABLED)
    is_delete = models.CharField(_(u'邮箱删除状态'), max_length=2, default='-1', choices=constants.MAILBOX_DISABLED)
    delete_time = models.DateTimeField(_(u'删除开始时间'), blank=True, null=True)
    ip_limit = models.CharField(_(u'只允许登录IP：'), max_length=255, null=True, blank=True)
    savepath = models.CharField(_(u'邮件保存地址：'), max_length=100, blank=True, null=True)
    limit_login = models.CharField(_(u'登录方式：'), max_length=2, default='-1', choices=constants.MAILBOX_LIMIT_LOGIN)
    recvsms = models.CharField(_(u'短信接收设置：'), max_length=2, choices=constants.MAILBOX_RECV_SMS, default='1')
    sys_mailbox = models.CharField(max_length=2, default='-1')
    change_pwd = models.CharField(_(u'登录强制修改密码：'), max_length=2, choices=constants.MAILBOX_CHANGE_PWD)
    enable_share = models.IntegerField(_(u'是否打开邮箱共享：'), default=-1, choices=constants.MAILBOX_ENABLE)
    # change_pwd = CharBooleanField(u'登录强制修改密码：', default=False)
    # enable_share = CharBooleanField(u'是否打开邮箱共享：', default=False)
    # first_change_pwd已经去掉
    # first_change_pwd = models.IntegerField(u'首次登录强制修改密码：', default=-1, choices=constants.MAILBOX_ENABLE)
    pwd_days = models.IntegerField(_(u'密码有效天数：'), default=7, help_text=_(u'大于0代表多少天密码过期后会强制用户修改密码,新增用户默认是X天.</br>'
                                                                         u'X天通过 <a target="_blank" href="/operation/security/password/rule">密码规则</a> 设置'))
    pwd_days_time = models.IntegerField(_(u'密码时间戳：'), default=int(time.time()), help_text=_(u'不能大于当前时间'))
    pwd_days_time2 = models.CharField(_(u'密码有效开始时间：'), max_length=64, default='', help_text=_(u'不能大于当前时间'))

    # 用户表 设置
    username = models.CharField(_(u'账号全名：'), max_length=200, null=False, blank=False, unique=True, db_column="mailbox")
    password = models.CharField(_(u'邮箱密码：'), max_length=120, blank=True, null=True)
    sms_password = models.CharField(u'客户端授权密码：', max_length=120, blank=True, null=True, help_text=u"仅允许客户端工具授权登录")
    sms_pwd_days = models.IntegerField(_(u'客户端授权码有效天数：'), default=0, help_text=_(u'大于0代表多少天后授权码过期失效,新增用户默认是X天.</br>'
                                                                         u'X天通过 <a target="_blank" href="/operation/security/password/rule">密码规则</a> 设置'))
    sms_pwd_days_time = models.DateTimeField(_(u'上次授权码修改时间'), blank=True, null=True)
    # 新增字段
    lock_password = models.CharField(u'文件夹密码', max_length=120, blank=True, null=True)
    last_login = models.DateTimeField(_(u'最后登录时间'), blank=True, null=True)
    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=30, blank=True)
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)
    # is_active = models.BooleanField(u'是否是活跃用户', default=False, help_text=u'不选择的话，被当作标记删除用户对待。此处用于区分 PHP前端用户，还是管理员用户')
    # is_active 不选择的话，被当作标记删除用户对待。此处用于区分 PHP前端用户，还是管理员用户
    is_active = models.BooleanField(_(u'管理员：'), default=False)
    is_staff = models.BooleanField(_(u'是否是员工'), default=True)
    is_superuser = models.BooleanField(
        _(u'超级管理员：'),
        default=False,
        # help_text=_(u'指定该用户具有所有权限。'),
    )
    user_permissions = models.ManyToManyField(
        Permission,
        verbose_name='user permissions',
        blank=True,
        help_text='Specific permissions for this user.',
        related_name="user_set",
        related_query_name="user",
        through='MyUserPermissions',
    )
    groups = models.ManyToManyField(
        Group,
        verbose_name=_('groups'),
        blank=True,
        help_text=_(
            'The groups this user belongs to. A user will get all permissions '
            'granted to each of their groups.'
        ),
        related_name="user_set",
        related_query_name="user",
        through='MyUserGroups',
    )

    domains = models.ManyToManyField(
        Domain,
        verbose_name=_(u'域名'),
        blank=True,
        related_name="domain_set",  # 比如 得到一个Tag模型tag， 查询 tag.tag_set.all()
        related_query_name="mailbox",  #
        through='UserDomain'
    )

    departments = models.ManyToManyField(
        Department,
        verbose_name=_(u'部门'),
        blank=True,
        related_name="department_set",  # 比如 得到一个Tag模型tag， 查询 tag.tag_set.all()
        related_query_name="mailbox",  #
        through='UserDepartment'
    )

    def is_otp_enabled(self):
        """
        Determine if a user has MFA enabled
        """
        return hasattr(self, 'userotp')


    def is_phone_enabled(self):
        """
        Phone if a user has MFA enabled
        """
        return licence_validsms() and hasattr(self, 'userphone')


    objects = UserManager()
    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = []

    class Meta:
        db_table = 'core_mailbox'
        managed = False
        verbose_name = _(u'邮箱用户表')

    def __unicode__(self):
        return self.username

    #############################################################
    # 权限
    # def get_all_permissions(self, obj=None):
    #     return _user_get_all_permissions(self, obj)

    def has_perm(self, perm, obj=None):
        """
        Returns True if the user has the specified permission. This method
        queries all available auth backends, but returns immediately if any
        backend returns True. Thus, a user who has permission from a single
        auth backend is assumed to have permission in general. If an object is
        provided, permissions for this specific object are checked.
        """

        # Active superusers have all permissions.
        # if self.is_active and self.is_superuser:
        #     return True

        # Otherwise we need to check the backends.
        return _user_has_perm(self, perm, obj)

    # def has_perms(self, perm_list, obj=None):
    #     """
    #     Returns True if the user has each of the specified permissions. If
    #     object is passed, it checks if the user has all required perms for this
    #     object.
    #     """
    #     return all(self.has_perm(perm, obj) for perm in perm_list)

    def has_module_perms(self, app_label):
        """
        Returns True if the user has any permissions in the given app label.
        Uses pretty much the same logic as has_perm, above.
        """
        # Active superusers have all permissions.
        # if self.is_active and self.is_superuser:
        #     return True

        return _user_has_module_perms(self, app_label)

    #############################################################

    def get_full_name(self):
        return self.username

    def get_short_name(self):
        return self.username

    def save(self, *args, **kwargs):
        # LINUX-1823 密码修改后, 使用旧密码登陆的在线用户应强制退出
        # 如果密码有修改过 记录修改时间 保存到redis里
        if self.id:
            new_password = getattr(self, 'password')
            self.refresh_from_db(fields=('password',))
            old_password = getattr(self, 'password')
            if old_password and new_password != old_password:
                redis = get_redis_connection()
                key = "pwd_modify_log"
                redis.hset(key, self.id, int(time.time()))
            setattr(self, 'password', new_password)
        super(Mailbox, self).save(*args, **kwargs)

    def check_password(self, raw_password, t_password=None):
        """
        Returns a boolean of whether the raw_password was correct. Handles
        hashing formats behind the scenes.
        """
        obj = Mailbox.objects.filter(username=self.username, disabled='-1').first()
        if not obj:
            return False
        succ = md5_crypt.verify(raw_password, self.password) if not t_password else t_password == self.password
        if not succ:
            return False
        # 判断是否要强制修改密码
        from app.utils.MailboxPasswordChecker import CheckMailboxPassword
        ret, reason = CheckMailboxPassword(domain_id=obj.domain_id, mailbox_id=obj.id, password=raw_password)
        if ret != 0:
            obj.change_pwd = '1'
            obj.save()
        return True

    def set_password(self, raw_password):
        self.password = md5_crypt.hash(raw_password)
        self.pwd_days_time = int(time.time())
        self.pwd_days_time2 = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(self.pwd_days_time))
        self.clear_reset_password()
        if self.id > 0:
            objAttr, created = MailboxUserAttr.objects.get_or_create(mailbox_id=self.id, domain_id=self.domain_id,
                                                                     type=u"system", item=u"password")
            raw_password = u"hHFdxF43et:::" + raw_password + u":::hHFdxF43et"
            raw_password = base64.encodestring(raw_password.encode("utf-8"))
            objAttr.value = raw_password
            objAttr.save()

    def clear_reset_password(self):
        CoreFindPassword.objects.filter(mailbox_id=self.id).delete()
    
    def get_pwd_days_time_display(self):
        try:
            val = time.localtime(int(self.pwd_days_time))
            return time.strftime('%Y-%m-%d %H:%M:%S', val)
        except:
            pass
        return ""

    @property
    def user(self):
        return MailboxUser.objects.filter(mailbox_id=self.id).first()

    @property
    def depts(self):
        return Department.objects.filter(
            id__in=DepartmentMember.objects.filter(mailbox_id=self.id).values_list('dept_id', flat=True))

    @property
    def size(self):
        # return MailboxSize.objects.filter(name=self.username).first()
        return MailboxSize.objects.filter(mailbox=self).first()

    @property
    def department(self):
        obj = DepartmentMember.objects.filter(mailbox_id=self.id).first()
        if obj:
            obj2 = Department.objects.filter(id=obj.dept_id).first()
            if obj2:
                return obj2.get_title
        return ""

    @property
    def open_distribute(self):
        # 通过测试开关开打开显示界面
        # insert into core_config(function,enabled) values('open_distribute','1');
        # delete from core_config where function='open_distribute';
        obj = CoreConfig.objects.filter(function="open_distribute").first()
        if obj and obj.enabled == '1':
            return True
        else:
            return False

    @property
    def open_custom_kkserver(self):
        obj = Domain.objects.filter(Q(domain="jinwan.gov.cn")).first()
        if not obj:
            return False
        return True

    @property
    def super_admin_title(self):
        obj = DomainAttr.objects.filter(domain_id=0, type="system", item=u'superadmintitle').first()
        if obj:
            return obj.value
        return _(u'福洽U-Mail安全增强电子邮件系统V9.8-管理员后台')

    @property
    def enable_remark(self):
        # 通过测试开关开打开显示界面
        # insert into core_config(function,enabled) values('enable_remark','1');
        # delete from core_config where function='enable_remark';
        obj = CoreConfig.objects.filter(function="enable_remark").first()
        if obj and obj.enabled == '1':
            return True
        else:
            return False

    @property
    def licence_validate(self):
        return licence_validate()

    @property
    def licence_validsms(self):
        # insert into core_config(function,enabled) values('enable_sms','1');
        # delete from core_config where function='enable_sms';
        obj = CoreConfig.objects.filter(function="enable_sms").first()
        if obj and obj.enabled == '1':
            return True
        return licence_validsms()

    @property
    def is_display_permission(self):
        from ConfigParser import ConfigParser
        cfg = ConfigParser()
        cfg.read('/usr/local/u-mail/config/custom.conf')
        if cfg.has_option('system', 'display_permission'):
            flag = cfg.get('system', 'display_permission')
            return str(flag).strip() == "1"
        return False

    @property
    def is_display_iparea(self):
        from ConfigParser import ConfigParser
        cfg = ConfigParser()
        cfg.read('/usr/local/u-mail/config/custom.conf')
        if cfg.has_option('system', 'display_iparea'):
            flag = cfg.get('system', 'display_iparea')
            return str(flag).strip() == "1"
        return True
        # return False

    @property
    def is_security_mode(self):
        return is_security_mode()

    @property
    def is_super_admin(self):
        # 安全模式下，没有超级管理员
        if self.is_security_mode:
            return False
        return self.is_superuser

    @property
    def is_security_admin(self):
        # 非安全模式下，系统管理员都是安全管理员
        if not self.is_security_mode:
            return self.is_superuser or self.is_sys_admin

        group = Group.objects.filter(name=u"安全保密管理员").first()
        if not group:
            return False
        users = group.user_set.filter(is_active=True).filter(id=self.id).first()
        return True if users else False

    @property
    def is_review_admin(self):
        # 非安全模式下，系统管理员都是安全管理员
        if not self.is_security_mode:
            return self.is_superuser or self.is_sys_admin

        group = Group.objects.filter(name=u"安全审计员").first()
        if not group:
            return False
        users = group.user_set.filter(is_active=True).filter(id=self.id).first()
        return True if users else False

    @property
    def is_sys_admin(self):
        if self.is_superuser:
            return False
        group = Group.objects.filter(name=u"系统管理员").first()
        if not group:
            return False
        users = group.user_set.filter(is_active=True).filter(id=self.id).first()
        if users:
            return True
        for group in Group.objects.filter(name__contains='----1'):
            if group.user_set.filter(is_active=True).filter(id=self.id).first():
                return True
        return False


    @property
    def is_domain_admin(self):
        if self.is_superuser:
            return False
        if self.is_sys_admin:
            return False
        group = Group.objects.filter(name=u"域名管理员").first()
        if not group:
            return False
        users = group.user_set.filter(is_active=True).filter(id=self.id).first()
        if users:
            return True
        for group in Group.objects.filter(name__contains='----3'):
            if group.user_set.filter(is_active=True).filter(id=self.id).first():
                return True
        return False
        return True if users else False

    @property
    def is_dept_admin(self):
        if self.is_superuser:
            return False
        # 域名管理员的优先级比部门管理员高
        if self.is_domain_admin:
            return False
        if self.is_sys_admin:
            return False
        # 部门管理员组
        group = Group.objects.filter(name=u"部门管理员").first()
        if group and group.user_set.filter(is_active=True).filter(id=self.id).first():
            return True
        from app.perm.models import MyPermission
        # 自定义组是否可以看到部门
        perm_list = [p.permission_id for p in
                     MyPermission.objects.filter(name__in=(u"dpt", u"department_list", "domain_group"))]
        for group in Group.objects.exclude(name__in=(u"部门管理员", u"域名管理员", u"系统管理员")):
            if not group.permissions.filter(id__in=perm_list).first():
                continue
            users = group.user_set.filter(is_active=True).filter(id=self.id).first()
            if users:
                return True
        return False

    @property
    def get_manage_domains(self):
        if self.is_superuser or self.is_sys_admin:
            return [(d.id, d.domain) for d in Domain.objects.filter(disabled='-1').all()]
        lst = self.domains.values_list('id', flat=True)
        return [(d.id, d.domain) for d in Domain.objects.filter(disabled='-1', id__in=lst).all()]

    @property
    def get_department_lst_in_page(self):
        if not self.is_dept_admin:
            return self.departments.all()
        else:
            group = Group.objects.filter(name=u"部门管理员").first()
            if not group:
                return []
            check_depts = [d.department_id for d in UserDepartment.objects.filter(user=self)]
            for d in self.depts:
                if not d.id in check_depts:
                    check_depts.append(d.id)
            return Department.objects.filter(id__in=check_depts)

    @property
    def get_department_manage_lst(self):
        group = Group.objects.filter(name=u"部门管理员").first()
        if not group:
            return []
        check_depts = [d.department_id for d in UserDepartment.objects.filter(user=self)]
        for d in self.depts:
            if not d.id in check_depts:
                check_depts.append(d.id)
        all_depts = []
        while check_depts:
            dept_id = check_depts.pop(0)
            if dept_id not in (0, -1) and dept_id not in all_depts:
                all_depts.append(dept_id)
                # 获取子部门
                lst = [d.id for d in Department.objects.filter(parent_id=dept_id)]
                check_depts.extend(lst)
        return Department.objects.filter(id__in=all_depts)

    @property
    def get_host_page(self):
        # if self.is_superuser or self.is_sys_admin:
        #     return u""
        # 按照  域名配置--邮箱管理--部门列表--邮件列表--日志/报告 顺序来显示
        uri_pages = [
            u"/operation/",
            u"/operation/domain/",
            u"/operation/mailbox/account",
            u"/operation/dpt/",
            u"/operation/maillist/",
            u"/operation/rpt/maillog/",
            u"/operation/rpt/user/log/",
            u"/operation/rpt/admin/log/",
        ]
        from app.perm.models import MyPermission
        for uri in uri_pages:
            permobj = MyPermission.objects.filter(url=uri).first()
            if not permobj:
                continue
            if self.has_perm(permobj.get_perm()):
                if uri == '/operation/':
                    return ''
                return uri
        for permobj in MyPermission.objects.all():
            if self.has_perm(permobj.get_perm()):
                if permobj.url == '/operation/':
                    return ''
                return permobj.url
        return u""

    @property
    def is_develop_server(self):
        obj = Domain.objects.filter(Q(domain="test.com") | Q(domain="domain.com")).first()
        if not obj:
            return False
        return True

    @property
    def get_delete_time_desc(self):
        if not self.delete_time:
            return _(u"等待删除")
        delay_setting = DomainAttr.getAttrObjValue(domain_id=0, type="system", item='cf_mailbox_delete_delay')
        delete_value = (self.delete_time + datetime.timedelta(days=int(delay_setting))).strftime('%Y-%m-%d %H:%M:%S')
        return _(u"将于<{}>后执行删除").format(delete_value)

    # 新版函数
    @property
    def getSendLimit(self):
        if hasattr(self, "tmp_limit_send"):
            return self.tmp_limit_send
        from app.utils.MailboxBasicChecker import MailboxBasicChecker
        group = MailboxBasicChecker(domain_id=self.domain_id, mailbox_id=self.id, mailbox=self.username)
        value = str(group.limit_send)
        value = "-1" if (not value or str(value) == "0") else str(value)
        self.tmp_limit_send = value
        return self.tmp_limit_send

    # 新版函数
    @property
    def getRecvLimit(self):
        if hasattr(self, "tmp_limit_recv"):
            return self.tmp_limit_recv
        from app.utils.MailboxBasicChecker import MailboxBasicChecker
        group = MailboxBasicChecker(domain_id=self.domain_id, mailbox_id=self.id, mailbox=self.username)
        value = str(group.limit_recv)
        value = "-1" if (not value or str(value) == "0") else str(value)
        self.tmp_limit_recv = value
        return self.tmp_limit_recv

    @property
    def is_resv_send_use_group(self):
        from app.group.models import CoreGroupMember, CoreGroup, CoreGroupSetting
        groupMember = CoreGroupMember.objects.filter(mailbox_id=self.id).order_by('id').first()
        if groupMember:
            setting = CoreGroupSetting.objects.filter(group_id=groupMember.group_id, type=u"recv_send").first()
            if setting:
                return True
        return False

    # 与组权限里面一起组合获取用户的邮箱容量
    @property
    def get_quota_mailbox(self):
        if hasattr(self, "tmp_quota_mailbox"):
            return self.tmp_quota_mailbox
        from app.utils.MailboxBasicChecker import MailboxBasicChecker
        group = MailboxBasicChecker(domain_id=self.domain_id, mailbox_id=self.id, mailbox=self.username)
        value = int(group.quota_mailbox)
        value = 0 if (not value or str(value) == "0") else int(value)
        self.tmp_quota_mailbox = value
        return self.tmp_quota_mailbox

    # 与组权限里面一起组合获取用户的网盘容量
    @property
    def get_quota_netdisk(self):
        if hasattr(self, "tmp_quota_netdisk"):
            return self.tmp_quota_netdisk
        from app.utils.MailboxBasicChecker import MailboxBasicChecker
        group = MailboxBasicChecker(domain_id=self.domain_id, mailbox_id=self.id, mailbox=self.username)
        value = int(group.quota_netdisk)
        value = 0 if (not value or str(value) == "0") else int(value)
        self.tmp_quota_netdisk = value
        return self.tmp_quota_netdisk

    @property
    def is_quota_use_group(self):
        box, disk = self.get_quota_mailbox, self.get_quota_netdisk
        if box != self.quota_mailbox or disk != self.quota_netdisk:
            return True
        return False

    @property
    def get_group_name(self):
        from app.group.models import CoreGroupMember, CoreGroup
        if hasattr(self, "tmp_group_name"):
            return getattr(self, "tmp_group_name")
        name = u""
        groupMember = CoreGroupMember.objects.filter(mailbox_id=self.id).order_by('id').first()
        if groupMember:
            group = CoreGroup.objects.filter(id=groupMember.group_id).first()
            if group:
                name = group.name
        setattr(self, "tmp_group_name", name)
        return name

    @property
    def display_limit_send(self):
        value = self.getSendLimit
        value = "-1" if (not value or str(value) == "0") else str(value)
        return dict(constants.MAILBOX_SEND_PERMIT)[value]

    @property
    def display_limit_recv(self):
        value = self.getRecvLimit
        value = "-1" if (not value or str(value) == "0") else str(value)
        return dict(constants.MAILBOX_RECV_PERMIT)[value]

    # 老版函数，新的已经不用，留作兼容的
    @property
    def getSendLimitWhiteList(self):
        lists = CoreWhitelist.objects.filter(type="fix_send", domain_id=self.domain_id, mailbox_id=self.id).all()
        num = 1
        for d in lists:
            yield num, d.id, d.email, str(d.disabled)
            num += 1

    # 老版函数，新的已经不用，留作兼容的
    @property
    def getRecvLimitWhiteList(self):
        lists = CoreWhitelist.objects.filter(type="fix_recv", domain_id=self.domain_id, mailbox_id=self.id).all()
        num = 1
        for d in lists:
            yield num, d.id, d.email, str(d.disabled)
            num += 1

    @property
    def is_old_version_webmail(self):
        if hasattr(settings, "OLD_WEBMAIL_VERSION") and settings.OLD_WEBMAIL_VERSION:
            return True
        return False

    def get_origin_password(self):
        password = ""
        obj = MailboxUserAttr.objects.filter(mailbox_id=self.id, type=u'system', item=u'password').first()
        if obj:
            password = obj.value
            # webmail对原始密码做了简单加密 key1xxxkey2
            serial_key_1 = "hHFdxF43et:::"
            serial_len_1 = len(serial_key_1)
            serial_key_2 = ":::hHFdxF43et"
            serial_len_2 = len(serial_key_2)

            password = base64.decodestring(password)
            password = password[serial_len_1: -serial_len_2]
        return password

    @property
    def jwt_expiration_delta(self):
        """ 用户session的过期时间 """
        if is_security_mode():
            v = DomainAttr.getAttrObjValue(domain_id=self.domain_id, type="system", item="cf_user_login_timeout")
            if (not v) or (v not in ("900", "1800", "3600", "5400", "7200", "10800", "21600", "43200")):
                return 1800
            try:
                return int(v)
            except:
                return 1800
        else:
            v = MailboxUserAttr.getObjV(self, self.domain, type="user", item="login_overtime", value="7200")
            if (not v) or (v not in ("1800", "3600", "7200", "18000", "28800", "86400", "604800")):  # "300", "600",
                return 7200
            try:
                return int(v)
            except:
                return 7200

    @property
    def get_admin_login_ip(self):
        try:
            obj = MailboxUserAttr.objects.filter(mailbox_id=self.id, type=u'system', item=u'admin_limit').first()
            if obj:
                d = json.loads(obj.value)
                enable = d.get("enable_ip", "0")
                if enable == "0":
                    return ""
                return d.get("ip", "").strip()
            return ""
        except:
            return ""

    @property
    def get_admin_login_time(self):
        try:
            obj = MailboxUserAttr.objects.filter(mailbox_id=self.id, type=u'system', item=u'admin_limit').first()
            if obj:
                d = json.loads(obj.value)
                enable = d.get("enable_time", "0")
                if enable == "0":
                    return "", ""
                start = d.get("time_start", "").strip()
                end = d.get("time_end", "").strip()
                return start, end
            return "", ""
        except:
            return "", ""

    def set_admin_limit(self, v):
        if isinstance(v, dict):
            v = json.dumps(v)
        o = MailboxUserAttr.getObj(self, self.domain, type=u"system", item=u"admin_limit")
        o.value = v
        o.save()

    def get_admin_limit(self):
        obj = MailboxUserAttr.objects.filter(mailbox_id=self.id, type=u'system', item=u'admin_limit').first()
        if not obj:
            return {}
        try:
            d = json.loads(obj.value)
            d = {} if not d else d
        except:
            d = {}
        return d

    @property
    def get_admin_login_error_count(self):
        return 5, 30 * 60

    def valid_admin_login(self, request):
        if not self.is_active:
            return u'非管理员禁止登陆'
        if not is_security_inited() and not self.is_security_admin:
            return u'服务器未经过安全管理员初始化，请先完成密级初始化'
        client_ip = get_client_ip(request)
        limit_ip = self.get_admin_login_ip
        ip_list = get_ip_info(limit_ip)
        if ip_list and not client_ip in ip_list:
            errmsg = ugettext(u'系统禁止管理员从此IP（%(client_ip)s）登陆。') % {'client_ip': client_ip}
            return errmsg
        start, end = self.get_admin_login_time
        start1, end1 = None, None
        try:
            if start and end:
                start1 = trans_time2sec(start)
                end1 = trans_time2sec(end)
        except:
            pass
        now = trans_time2sec(time.strftime("%H:%M:%S"))
        if (start1 is not None and end1 is not None) and (now < start1 or now > end1):
            errmsg = ugettext(u'系统禁止管理员从该时段（允许%(start)s-%(end)s)登陆。') % {'start': start, 'end': end}
            return errmsg
        # 验证账号是否因为错误登陆被禁用了
        key = "linux-webvue:core:login:limit:{}:{}".format(self.id, client_ip)
        redis = get_redis_connection()
        error_count, error_seconds = self.get_admin_login_error_count
        if redis.get(key) and int(redis.get(key)) >= error_count and redis.ttl(key) > 0:
            left_seconds = redis.ttl(key)
            if int(left_seconds) <= 60:
                left_seconds = u"{}秒".format(left_seconds)
            else:
                left_seconds2 = left_seconds % 60
                left_seconds = left_seconds / 60
                left_seconds = u"{}分{}秒".format(left_seconds, left_seconds2)
            errmsg = ugettext(u'账号错误登陆次数过多，系统禁止登陆（剩余%(left_seconds)s）。') % {'left_seconds': left_seconds}
            return errmsg
        return u""

    @property
    def get_mail_permission(self):
        """获取邮件及网盘操作权限，没有值时默认开启"""
        # 网盘下载
        obj1, create1 = MailboxUserAttr.objects.get_or_create(mailbox_id=self.id, domain_id=self.domain_id,
                                                              type=u"user", item=u"sw_netdisk_download")
        if create1:
            obj1.value = u"1"
            obj1.save()
        # 邮件转发
        obj2, create2 = MailboxUserAttr.objects.get_or_create(mailbox_id=self.id, domain_id=self.domain_id,
                                                              type=u"user", item=u"sw_mail_forward")
        if create2:
            obj2.value = u"1"
            obj2.save()
        # 邮件下载
        obj3, create3 = MailboxUserAttr.objects.get_or_create(mailbox_id=self.id, domain_id=self.domain_id,
                                                              type=u"user", item=u"sw_mail_download")
        if create3:
            obj3.value = u"1"
            obj3.save()
        return obj1.value, obj2.value, obj3.value

    @property
    def get_sw_hiding_not_default_folder(self):
        # 广东浩传管理服务有限公司 邮箱网页端[我的文件夹]定制化设置:
        obj, create = MailboxUserAttr.objects.get_or_create(mailbox_id=self.id, domain_id=self.domain_id,
                                                              type=u"user", item=u"sw_hiding_not_default_folder")
        if create:
            obj.value = u"-1"
            obj.save()
        return obj.value

    @property
    def get_mail_permission_desc(self):
        lst = []
        perm1, perm2, perm3 = self.get_mail_permission
        if perm1 == "1":
            lst.append(_(u"网盘下载"))
        if perm2 == "1":
            lst.append(_(u"转发邮件"))
        if perm3 == "1":
            lst.append(_(u"下载邮件"))
        if not lst:
            return _(u"无任何权限")
        lst = map(lambda s:str(s).decode('utf-8'), lst)
        return u"|".join(lst).encode('utf-8')

    @property
    def is_add_account_must_dept(self):
        try:
            import ConfigParser
            cfg_obj = ConfigParser.ConfigParser()
            cfg_obj.read("/usr/local/u-mail/app/conf/setting.conf")
            must = 0
            if cfg_obj.has_section("extra"):
                if cfg_obj.has_option("extra", "account_must_dept"):
                    must = int(cfg_obj.get('extra', 'account_must_dept', '0').strip())
            return must == 1
        except Exception, err:
            return False


class CoreFindPassword(models.Model):
    """ 找回密码 管理员重置的时候需要清空此项"""
    mailbox = models.OneToOneField(Mailbox, related_name='userfindpasswd')
    code = models.CharField(u"验证码", max_length=4, null=True, blank=True)
    count = models.IntegerField(u"当天验证次数", default=0)
    carbled = models.CharField(u"唯一乱码，uuid", max_length=100, null=True, db_index=True)
    new_carbled = models.CharField(u"唯一乱码，密码验证", max_length=100, null=True, db_index=True)
    expired = models.IntegerField(u"过期时间", default=0)
    expired_long = models.IntegerField(u"过期时间，一天之内的过期时间", default=0)

    class Meta:
        managed = False
        db_table = 'core_user_find'


class MailboxUser(models.Model):
    """
    邮件账号信息
    """
    # id = models.IntegerField(primary_key=True, db_column='mailbox_id')
    mailbox = models.OneToOneField(Mailbox, primary_key=True, related_name='mailboxuser')
    domain = models.ForeignKey(Domain)

    realname = models.CharField(_(u'姓名：'), max_length=80, null=True, blank=True)
    eenumber = models.CharField(_(u'工号：'), max_length=200, null=True, blank=True)
    last_login = models.DateTimeField(_(u'最后登录'), blank=True, null=True)
    engname = models.CharField(_(u'英文名：'), max_length=35, blank=True, null=True)
    oabshow = models.CharField(_(u'通讯录显示：'), max_length=2, default='1', choices=constants.USER_SHOW)
    # oabshow = CharBooleanField(u'通讯录显示：', default=True)
    showorder = models.IntegerField(_(u'排序权重：'), default=0, help_text=_(u'注：数字越大越靠前显示'))
    gender = models.CharField(_(u'性别：'), max_length=6, choices=constants.GENDER, default='male')
    birthday = ZeroDateField(_(u'生日：'), null=True, blank=True)
    homepage = models.CharField(_(u'主页：'), max_length=100, blank=True, null=True)
    tel_mobile = models.CharField(_(u'手机号码：'), max_length=20, blank=True, null=True)
    tel_home = models.CharField(_(u'住宅号码：'), max_length=20, blank=True, null=True)
    tel_work = models.CharField(_(u'公司电话：'), max_length=20, blank=True, null=True)
    tel_work_ext = models.CharField(_(u'分机号码：'), max_length=10, blank=True, null=True)
    tel_group = models.CharField(_(u'集团号：'), max_length=20, blank=True, null=True)
    im_qq = models.CharField(u'QQ：', max_length=25, blank=True, null=True)
    im_msn = models.CharField(u'MSN：', max_length=50, blank=True, null=True)
    addr_country = models.CharField(max_length=50, blank=True, null=True)
    addr_state = models.CharField(max_length=50, blank=True, null=True)
    addr_city = models.CharField(max_length=50, blank=True, null=True)
    addr_address = models.CharField(max_length=100, blank=True, null=True)
    addr_zip = models.CharField(max_length=20, blank=True, null=True)
    remark = models.TextField(_(u'备注：'), blank=True, null=True)
    last_session = models.CharField(max_length=32, blank=True, null=True)
    openid = models.CharField(max_length=128, default='0')
    unionid = models.CharField(max_length=255, default='0')
    wx_id = models.BigIntegerField(default=0)

    class Meta:
        db_table = 'co_user'
        managed = False
        verbose_name = _(u'用户信息表')


class MailboxUserAttr(models.Model):
    mailbox = models.ForeignKey(Mailbox, on_delete=models.CASCADE)
    domain = models.ForeignKey(Domain)
    type = models.CharField(_(u"类型"), max_length=6)
    item = models.CharField(_(u"变量"), max_length=35, blank=True, null=True)
    value = models.TextField(_(u"值"), blank=True, null=True)

    class Meta:
        managed = False
        db_table = 'co_user_attr'
        unique_together = (('mailbox', 'type', 'item'),)

    @staticmethod
    def getObj(mailbox, domain, type="user", item=None):
        obj, _created = MailboxUserAttr.objects.get_or_create(mailbox=mailbox, domain=domain, type=type, item=item)
        return obj

    @staticmethod
    def getObjV(mailbox, domain, type="user", item=None, value=None):
        """
        :param mailbox:
        :param domain:
        :param type:
        :param item:
        :param value: 加入没有此值，则使用默认值value并设置进去
        :return:
        """
        obj = MailboxUserAttr.getObj(mailbox, domain, type, item)
        if not obj.value and value:
            obj.value = value
            obj.save()
        return obj.value


REGISTER_CHOICE = (
    (u"wait", _(u"等待处理")),
    (u"permit", _(u"批准")),
    (u"reject", _(u"拒绝")),
)


class CoUserReg(models.Model):
    domain_id = models.IntegerField()
    username = models.CharField(max_length=35, blank=True, null=True)
    password = models.CharField(max_length=40, blank=True, null=True)
    realname = models.CharField(max_length=35, blank=True, null=True)
    engname = models.CharField(max_length=35, blank=True, null=True)
    eenumber = models.CharField(max_length=20, blank=True, null=True)
    mobile = models.CharField(max_length=20, blank=True, null=True)
    department = models.IntegerField()
    remark = models.TextField(blank=True, null=True)
    status = models.CharField(max_length=6, choices=REGISTER_CHOICE)
    created = models.DateTimeField()

    class Meta:
        managed = False
        db_table = 'co_user_reg'
        verbose_name = _(u'用户注册表')

    @property
    def get_created_time(self):
        try:
            return self.created.strftime("%Y-%m-%d %H:%M:%S")
        except:
            return u""

    @property
    def get_department_name(self):
        if not self.department:
            return u""
        obj = Department.objects.filter(id=self.department).first()
        if obj:
            return obj.title
        return u""


REVIEW_CHOICE = (
    (u"wait", _(u"等待处理")),
    (u"permit", _(u"已批准")),
    (u"reject", _(u"已拒绝")),
)


class ExtPasswordReview(models.Model):
    """docstring for ExtPasswordReview"""
    mailbox_id = models.IntegerField()
    username = models.CharField(max_length=200, blank=False, null=False)
    admin = models.CharField(max_length=200, blank=False, null=False)
    new_password = models.CharField(u'new_password', max_length=200, null=False, blank=False, unique=True,
                                    help_text=u'新密码')
    modify_time = models.DateTimeField()
    status = models.CharField(max_length=6, choices=REVIEW_CHOICE)

    class Meta:
        db_table = 'ext_password_review'
        managed = False
        verbose_name = _(u'密码审核表')

    @property
    def get_modify_time(self):
        try:
            return self.modify_time.strftime("%Y-%m-%d %H:%M:%S")
        except:
            return u""


class UserDomain(models.Model):
    user = models.ForeignKey(Mailbox, verbose_name=_(u"用户"))
    domain = models.ForeignKey(Domain, verbose_name=_(u"域名"))

    class Meta:
        managed = False
        auto_created = True
        db_table = 'co_user_domain'
        verbose_name = _(u'用户域名关联')
        unique_together = (
            ('user', 'domain'),
        )


class UserDepartment(models.Model):
    user = models.ForeignKey(Mailbox, verbose_name=_(u"用户"))
    department = models.ForeignKey(Department, verbose_name=_(u"部门"))

    class Meta:
        managed = False
        auto_created = True
        db_table = 'co_user_department'
        verbose_name = _(u'用户部门关联')
        unique_together = (
            ('user', 'department'),
        )


class DepartmentMember(models.Model):
    """
    部门信息
    """
    domain = models.ForeignKey(Domain, null=False, blank=False, verbose_name=_(u"域名"))
    dept_id = models.IntegerField(_(u'部门ID'), default=-1)
    mailbox_id = models.IntegerField(_(u'邮箱ID'), default=-1)
    position = models.CharField(_(u'职位'), max_length=35, null=True, blank=True)

    @property
    def dept(self):
        return Department.objects.filter(id=self.dept_id).first()

    class Meta:
        db_table = "co_department_member"
        managed = False
        verbose_name = _(u'部门成员表')


class MailboxSize(models.Model):
    """
    邮件账号信息
    """
    # id = models.AutoField(primary_key=True, db_column='mailbox_id')
    mailbox = models.OneToOneField(Mailbox, primary_key=True, related_name='mailboxsize')
    name = models.CharField(_(u'账号全名'), max_length=200, null=False, blank=False, db_column='mailbox')
    size = models.IntegerField(_(u'邮箱当前容量'), default=0)
    per = models.IntegerField(_(u'邮箱容量使用比例'), default=0)
    last_update = models.DateTimeField('update_time', blank=True, null=True)

    def __unicode__(self):
        return self.name

    class Meta:
        db_table = 'ext_mailbox_size'
        managed = False


class MyUserPermissions(models.Model):
    user = models.ForeignKey(Mailbox, verbose_name=_(u"用户"))
    permission = models.ForeignKey(Permission, verbose_name=_(u"权限"))

    class Meta(AbstractUser.Meta):
        managed = False
        auto_created = True
        db_table = 'auth_user_user_permissions'
        verbose_name = _(u'用户权限表')


class MyUserGroups(models.Model):
    user = models.ForeignKey(Mailbox, verbose_name=_(u"用户"))
    group = models.ForeignKey(Group, verbose_name=_(u"用户组"))

    class Meta(AbstractUser.Meta):
        managed = False
        auto_created = True
        db_table = 'auth_user_groups'
        verbose_name = _(u'用户权限组表')


class CoreUrlRemark(models.Model):
    """  每个页面下的备注， url唯一
    """
    url = models.CharField(u'URL', max_length=200, null=False, blank=False, unique=True, help_text=_(u'url地址'))
    remark = models.TextField(_(u'备注'), null=True, blank=True)

    class Meta:
        managed = False
        db_table = 'core_url_remark'


class CoreTrustIP(models.Model):
    """ 信任IP """
    ip = models.CharField(_(u'信任IP'), max_length=48, null=False, blank=False, db_index=True)
    auth = models.IntegerField(_(u'需要验证'), default=1, choices=constants.TRUST_IP_AUTH_STATUS)
    disabled = models.CharField(_(u'状态'), max_length=1, choices=constants.DISABLED_STATUS, default='-1', null=False,
                                blank=False)

    class Meta:
        db_table = "core_trustip"
        managed = False
        verbose_name = _(u'信任IP表')

    def __unicode__(self):
        return self.ip


class CoreAlias(models.Model):
    """ 邮件域别名 """
    domain_id = models.IntegerField(_(u'域名ID'), default=0, null=False, blank=False, db_index=True)
    mailbox_id = models.IntegerField(_(u'邮箱ID'), default=0, null=False, blank=False, db_index=True)
    type = models.CharField(_(u"类型"), max_length=16, choices=constants.CORE_ALIS_TYPE, default='mailbox', null=False,
                            blank=False)
    source = models.CharField(_(u"虚拟邮件域"), max_length=200, db_index=True)
    note = models.CharField(_(u"备注"), max_length=50, db_index=True)
    target = models.TextField(_(u"真实邮件域"))
    disabled = models.CharField(_(u'状态'), max_length=1, choices=constants.DISABLED_STATUS, default='-1', null=False,
                                blank=False)

    class Meta:
        db_table = "core_alias"
        managed = False
        verbose_name = _(u'别名表')

    def __str__(self):
        return self.source


class CoreBlacklist(models.Model):
    """黑名单 """
    operator = models.CharField(_(u"用户类型"), max_length=10, default="user", choices=constants.BLACK_WHITE_OPTOR,
                                null=False, blank=False)
    type = models.CharField(_(u"类型"), max_length=16, choices=constants.BLACK_WHITE_TYPE, default='recv', null=False,
                            blank=False)
    domain_id = models.IntegerField(_(u'域名ID'), default=0, null=False, blank=False)
    mailbox_id = models.IntegerField(_(u'邮箱ID'), default=0, null=False, blank=False)
    email = models.CharField(_(u"邮箱"), max_length=60, db_index=True)
    disabled = models.CharField(_(u'状态'), max_length=1, choices=constants.DISABLED_STATUS, default='-1', null=False,
                                blank=False)
    add_time = models.IntegerField(_(u"添加时间"), default=0, null=False, blank=False)
    remark = models.TextField(_(u"备注"), null=True, blank=True)

    class Meta:
        db_table = "core_blacklist"
        managed = False
        verbose_name = _(u'黑名单列表')

    def __str__(self):
        return self.email

    @property
    def addtime(self):
        return datetime.datetime.fromtimestamp(self.add_time).strftime("%Y-%m-%d %H:%M:%S")


class CoreWhitelist(models.Model):
    """ 白名单 """
    operator = models.CharField(_(u"用户类型"), max_length=10, default="user", choices=constants.BLACK_WHITE_OPTOR,
                                null=False, blank=False)
    type = models.CharField(_(u"类型"), max_length=16, choices=constants.BLACK_WHITE_TYPE, default='recv', null=False,
                            blank=False)
    # version = models.IntegerField("版本", default=1, null=False, blank=False)
    domain_id = models.IntegerField(_(u'域名ID'), default=0, null=False, blank=False)
    mailbox_id = models.IntegerField(_(u'邮箱ID'), default=0, null=False, blank=False)
    email = models.CharField(_(u"邮箱"), max_length=60, db_index=True)
    disabled = models.CharField(_(u'状态'), max_length=1, choices=constants.DISABLED_STATUS, default='-1', null=False,
                                blank=False)
    add_time = models.IntegerField(_(u"添加时间"), default=0, null=False, blank=False)
    remark = models.TextField(_(u"备注"), null=True, blank=True)

    class Meta:
        db_table = "core_whitelist"
        managed = False
        verbose_name = _(u'白名单列表')

    def __str__(self):
        return self.email

    @property
    def addtime(self):
        return datetime.datetime.fromtimestamp(self.add_time).strftime("%Y-%m-%d %H:%M:%S")


class DomainAttr(models.Model):
    domain_id = models.IntegerField(_(u"域名ID"), default=0, null=False, blank=False)
    type = models.CharField(_(u"类型"), choices=constants.ATTR_TYPR, max_length=20, null=False, blank=False)
    item = models.CharField(_(u"键"), max_length=35, null=False, blank=False)
    value = models.TextField(_(u"值"))

    class Meta:
        db_table = 'core_domain_attr'
        managed = False
        unique_together = (
            ('domain_id', 'type', 'item'),
        )
        verbose_name = _(u'域名设置表')

    def __str__(self):
        return self.value

    def __unicode__(self):
        return self.value

    @staticmethod
    def getAttrObj(domain_id=0, type="system", item=None):
        obj, _created = DomainAttr.objects.get_or_create(domain_id=domain_id, type=type, item=item)
        if _created:
            default = dict(constants.DOMAINATTR_DEFULT).get(item, '')
            # 安全模式下，默认值是禁止发送所有邮件
            if item in ("limit_send", "limit_recv") and is_security_mode():
                default = "1"
            obj.value = default
            obj.save()
        return obj

    @staticmethod
    def getAttrObjValue(domain_id=0, type="system", item=None):
        value = DomainAttr.getAttrObj(domain_id, type, item).value
        if not value:
            return dict(constants.DOMAINATTR_DEFULT).get(item, '')
        return value

    @staticmethod
    def saveAttrObjValue(domain_id=0, type="system", item=None, value=None):
        obj = DomainAttr.getAttrObj(domain_id, type, item)
        obj.value = value
        obj.save()

    @staticmethod
    def emptyAttrObjValue(domain_id=0, type="system", item=None):
        DomainAttr.saveAttrObjValue(domain_id, type, item, value="")

    @staticmethod
    def smtpSetValuetoDict():
        """ used for form initial
        """
        j = {"1th": "1", "2nd": "5", "3rd": "60"}
        value = DomainAttr.getAttrObjValue(item="cf_smtp_retry")
        if value:
            j = json.loads(value)
            j = {} if (not j or str(j) == "-1") else j
        return {
            "smtp_seta": j["1th"] if "1th" in j else "5",
            "smtp_setb": j["2th"] if "2th" in j else "25",
            "smtp_setc": j["3th"] if "3th" in j else "60",
        }

    @staticmethod
    def getSignatureCache():
        """ 证书签名请求 数据缓存
        """
        j = {'sig_depart': '', 'sig_organization': '', 'sig_domain': '', 'sig_province': '', 'sig_locale': ''}
        value = DomainAttr.getAttrObjValue(item="ssl_signrequest_cache")
        if value:
            j = json.loads(value)
            j = {} if (not j or str(j) == "-1") else j
        return {
            "sig_depart": dict_compatibility(j, "sig_depart", ""),
            "sig_organization": dict_compatibility(j, "sig_organization", ""),
            "sig_domain": dict_compatibility(j, "sig_domain", ""),
            "sig_province": dict_compatibility(j, "sig_province", ""),
            "sig_locale": dict_compatibility(j, "sig_locale", ""),
        }


CONFIG_DISABLED_STATUS = (
    ('-1', _(u'禁用')),
    ('1', _(u'启用')),
)


class CoreConfig(models.Model):
    function = models.CharField(_(u"类型"), max_length=50, null=True, blank=True)
    enabled = models.CharField(_(u'状态'), max_length=10, choices=CONFIG_DISABLED_STATUS, default='-1', null=False,
                               blank=False)
    param = models.TextField(_(u"值"))

    class Meta:
        db_table = 'core_config'
        managed = False
        verbose_name = _(u'系统设置')

    def __str__(self):
        return self.function

    @staticmethod
    # core_config最初的设计比较奇葩，同样的显示，保存在不同的数据库列
    def analyseFormParam(function, enabled):
        param = None
        if function == "recipientlimit":
            param = enabled
            enabled = "1"
        elif function == "notice_lang":
            if str(enabled) == "1":
                param = "chinese"
            else:
                param = "english"
        return enabled, param

    @staticmethod
    def analyseFormValue(function, default=''):
        if function == "recipientlimit":
            return CoreConfig.getFuctionParam(function, default)
        return CoreConfig.getFuctionEnabled(function, default)

    @staticmethod
    def getFuctionObj(function):
        obj, _created = CoreConfig.objects.get_or_create(function=function)
        if _created:
            obj.enabled = "-1"
            obj.save()
        return obj

    @staticmethod
    def getFuctionEnabled(function, default='-1'):
        obj = CoreConfig.getFuctionObj(function)
        if not obj.enabled:
            obj.enabled = default
            obj.save()
        return obj.enabled

    @staticmethod
    def getFuctionParam(function, default=''):
        obj = CoreConfig.getFuctionObj(function)
        if not obj.param:
            obj.param = default
            obj.save()
        return obj.param

    @staticmethod
    def saveFuction(function, enabled, param, withenabled=True):
        obj = CoreConfig.getFuctionObj(function)
        if withenabled:
            obj.enabled = enabled
        obj.param = param
        obj.save()
        return obj

    @staticmethod
    def getInitBackupParam(function="auto_backup"):
        """ used for backup form
        数据备份参数列表初始化
        """
        j = {
            # 保存路径
            "path": "/usr/local/u-mail/data/backup/",
            # 数据类型 database，
            "data": ["database", ],
            # 保留数量
            "count": "3",
            # 备份周期 hour，day，week，month，year
            "type": "day",
            "cycle": "1",
            # 备份时间, 根据 "备份周期" 的 年、月、日，填充以下五个字段
            # 类型参数：
            # 小时：  只输入  minute
            # 天： hour，minute
            # 周： week，hour，minute
            # 月： date,week，hour，minute
            # 年： month，date,week，hour，minute
            "month": "1",
            "date": "1",
            "week": "7",
            "hour": "4",
            "minute": "0",
            "mail_to": "",
        }

        param = CoreConfig.getFuctionParam(function=function)
        if param: j = json.loads(param)
        return {
            "path": dict_compatibility(j, "path", "/usr/local/u-mail/data/backup/"),
            "data": dict_compatibility(j, "data", ["database"]),
            "count": dict_compatibility(j, "count", 10),
            "type": dict_compatibility(j, "type", "day"),
            "cycle": dict_compatibility(j, "cycle", 1),

            "month": dict_compatibility(j, "month", ""),
            "date": dict_compatibility(j, "date", ""),
            "week": dict_compatibility(j, "week", ""),
            "hour": dict_compatibility(j, "hour", ""),
            "minute": dict_compatibility(j, "minute", "0"),
            "mail_to": dict_compatibility(j, "mail_to", ""),
        }


class CoreMonitor(models.Model):
    domain_id = models.IntegerField(_(u'域名ID'), default=0, null=False, blank=False, db_index=True)
    target = models.CharField(_(u"监控对象"), max_length=80, db_index=True)
    target_dept = models.IntegerField(_(u'发信人部门'), default=0, help_text=_(u'需要审核的部门ID'))
    forward = models.CharField(_(u"接收邮箱"), max_length=80, db_index=True)
    forward_folder = models.CharField(_(u"邮件存储路径"), max_length=80, default='', help_text=_(u'投递位置'))
    listen_type = models.CharField(_(u'监听类型'), max_length=20, choices=constants.MONITOR_LISTEN_TYPE, db_column='type')
    target_type = models.CharField(_(u'通道类型'), max_length=20, choices=constants.MONITOR_TARGET_TYPE)
    monit_move = models.CharField(_(u'监控邮件搬家'), max_length=10, choices=constants.MONITOR_MAILMOVE_SELECT)
    disabled = models.CharField(_(u'状态'), max_length=1, choices=constants.DISABLED_STATUS, default='-1', null=False,
                                blank=False)

    class Meta:
        db_table = 'core_bcc'
        managed = False
        verbose_name = _(u'邮箱监控表')

    @property
    def department(self):
        obj = Department.objects.filter(pk=self.target_dept).first()
        return obj and obj.get_title or ''

    def get_domain_name(self):
        obj = Domain.objects.filter(id=self.domain_id).first()
        if not obj:
            return ""
        return obj.domain


class VisitLog(models.Model):
    """Web 登录日志"""
    domain = models.ForeignKey(Domain, null=True, blank=True, db_index=True)
    mailbox = models.ForeignKey(Mailbox, null=False, blank=False, db_index=True)
    logintime = models.DateTimeField('login time', blank=True, null=True)
    lasttime = models.DateTimeField('last time', blank=True, null=True)
    clienttype = models.CharField('clienttype', max_length=32, null=True, blank=True)
    clientip = models.CharField('clientip', max_length=32, null=True, blank=True)

    class Meta:
        db_table = 'co_user_visitlog'
        managed = False
        verbose_name = _(u'用户访问log')


class AuthLog(models.Model):
    """
    客户端访问日志
    """
    domain_id = models.IntegerField(_(u'域名ID'), default=0, null=False, blank=False, db_index=True)
    user = models.CharField('user', max_length=100, null=True, blank=True)
    type = models.CharField('type', max_length=10, null=True, blank=True)
    client_ip = models.CharField('client_ip', max_length=20, null=True, blank=True)
    is_login = models.BooleanField(default=True)
    time = models.DateTimeField('time', blank=True, null=True)
    remark = models.TextField(null=True, blank=True)
    date = models.DateField(null=True, blank=True, editable=False)


    class Meta:
        db_table = 'core_auth_log'
        managed = False
        verbose_name = _(u'用户验证log')

    def save(self, *args, **kwargs):
        if isinstance(self.time, unicode):
            try:
                self.time = datetime.datetime.strptime(self.time, '%Y-%m-%d %H:%M:%S')
            except ValueError:
                self.time = None

        if self.time:
            self.date = self.time.date()
        # 调用父类的 save 方法时，确保不传递 date 字段
        self._meta.local_fields = [f for f in self._meta.local_fields if f.name != 'date']
        super(AuthLog, self).save(*args, **kwargs)
        self._meta.local_fields = [f for f in self._meta.local_fields]  # 恢复原状



class UpgradeList(models.Model):
    company = models.CharField('company', max_length=250, null=False, blank=True)
    domain = models.CharField('domain', max_length=100, null=False, blank=True)
    app = models.CharField('app', max_length=50, null=True, blank=True)
    prev_app = models.CharField('prev_app', max_length=50, null=True, blank=True)
    webmail = models.CharField('webmail', max_length=50, null=True, blank=True)
    prev_webmail = models.CharField('prev_webmail', max_length=50, null=True, blank=True)
    operation = models.CharField('operation', max_length=50, null=True, blank=True)
    prev_operation = models.CharField('prev_operation', max_length=50, null=True, blank=True)
    update_time = models.DateTimeField('update_time', blank=True, null=True)

    class Meta:
        db_table = 'core_upgrade_list'
        managed = False

    @staticmethod
    def getObj(domain):
        obj, _created = UpgradeList.objects.get_or_create(domain=domain)
        return obj


class MailboxExtra(models.Model):
    mailbox_id = models.IntegerField()
    mailbox = models.CharField(_(u"邮箱"), max_length=80)
    size = models.IntegerField(_(u"大小"), default=0)
    type = models.CharField(_(u"类型"), max_length=20)
    data = models.CharField(_(u"值"), max_length=500)
    last_update = models.DateTimeField(auto_now=True)

    class Meta:
        managed = False
        db_table = 'ext_mailbox_extra'
        unique_together = (('mailbox_id', 'type'),)
        verbose_name = _(u'用户信息表2')


class ExtCommonCheckrule(models.Model):
    mailbox = models.ForeignKey(Mailbox)
    type = models.CharField(_(u"类型"), max_length=50, blank=True, null=True, choices=constants.CHECKRULE_TYPE)
    logic = models.CharField(_(u"逻辑"), max_length=50, blank=True, null=True)
    disabled = models.IntegerField()

    def __unicode__(self):
        return ugettext(u"规则")

    @property
    def conditions(self):
        return self.rule_condition.filter(disabled=-1, parent_id=0)

    class Meta:
        managed = False
        db_table = 'ext_common_checkrule'
        verbose_name = _(u'规则')


class ExtCheckruleCondition(models.Model):
    parent_id = models.IntegerField(default=0)
    logic = models.CharField(_(u"逻辑"), max_length=50, choices=constants.CHECKRULE_CONDITION_LOGIC)
    rule = models.ForeignKey(ExtCommonCheckrule, related_name='rule_condition')
    option = models.CharField(_(u"条件"), max_length=50, blank=True, null=True,
                              choices=constants.CHECKRULE_CONDITION_OPTION)
    action = models.CharField(_(u"动作"), max_length=20, blank=True, null=True,
                              choices=constants.CHECKRULE_CONDITION_ACTION)
    value = models.CharField(_(u"值"), max_length=500, blank=True, null=True)
    disabled = models.IntegerField()

    def __unicode__(self):
        return ugettext(u"条件")

    @property
    def children(self):
        lists = [self]
        children = ExtCheckruleCondition.objects.filter(parent_id=self.id)
        if children:
            for child in children:
                lists.extend(child.children)
        return lists

    @property
    def json_value(self):
        return json.loads(self.value)

    class Meta:
        managed = False
        ordering = ['id']
        db_table = 'ext_checkrule_condition'
        verbose_name = _(u'条件')


class ExtReply(models.Model):
    mailbox = models.ForeignKey(Mailbox)
    domain = models.ForeignKey(Domain, related_name="reply_domain")
    address = models.CharField(_(u"邮箱"), max_length=200, blank=True, null=True)
    rule = models.ForeignKey(ExtCommonCheckrule)
    sequence = models.IntegerField(_(u"排序"), default=999)
    body = models.TextField(_(u"回复内容"), blank=True, null=True)
    work_mode = models.CharField(max_length=5, default='full', choices=constants.EXTRELAY_WORK_MODE)
    work_start = ZeroDateField(null=True, blank=True)
    work_end = ZeroDateField(null=True, blank=True)
    disabled = models.CharField(max_length=2, choices=constants.DISABLED_STATUS)

    class Meta:
        managed = False
        db_table = u'ext_reply'
        verbose_name = _(u'自动回复')


class ExtForward(models.Model):
    mailbox = models.ForeignKey(Mailbox)
    domain = models.ForeignKey(Domain, related_name="forward_domain")
    username = models.CharField(max_length=200, blank=True, null=True, db_column='mailbox')
    rule = models.ForeignKey(ExtCommonCheckrule)
    sequence = models.IntegerField(_(u"排序"), default=999)
    forward = models.TextField(blank=True, null=True)
    visible = models.CharField(_(u'用户可否设置'), max_length=2, default='-1', choices=constants.FORWARD_VISIBLE,
                               help_text=_(u'已弃用'))
    keep_mail = models.CharField(_(u'同时将信件保存在本邮箱内'), max_length=2, choices=constants.FORWARD_KEEP_MAIL)
    disabled = models.CharField(max_length=2, choices=constants.DISABLED_STATUS)
    local = models.CharField(_(u'仅转发本地邮箱'), max_length=2, default='-1', help_text=_(u'已弃用'))

    class Meta:
        managed = False
        db_table = 'ext_forward'


class ProxyRedisLog(models.Model):
    type = models.IntegerField(default=1)
    exec_type = models.CharField(max_length=10, default='add')
    data = models.CharField(max_length=300, blank=True, null=True)
    save_status = models.IntegerField(default=0)
    exec_status = models.IntegerField(default=0)
    protocol = models.CharField(max_length=50, blank=True, null=True)
    add_time = models.IntegerField(default=time.time())
    update_time = models.IntegerField(default=0)

    class Meta:
        managed = False
        db_table = 'proxy_redis_log'

class UserPhone(models.Model):
    user = models.OneToOneField(Mailbox, on_delete=models.PROTECT, related_name='userphone')
    phone = models.CharField(max_length=128, blank=False, null=False)

    class Meta:
        db_table = 'twofactor_phone'
        managed = False
        verbose_name = u'手机两步验证'

    def __unicode__(self):
        return self.phone

class MfaTrust(models.Model):
    ip = models.CharField(_(u"IP"), max_length=50, null=False, blank=True)
    name = models.CharField(_(u"备注"), max_length=200, null=True, blank=True)
    disabled = models.SmallIntegerField(_(u'激活状态'), choices=constants.DISABLED_STATUS, default=-1)

    class Meta:
        managed = False
        db_table = 'ext_mfa_trust'
        verbose_name = _(u'Mfa信任IP')

    def __unicode__(self):
        return self.ip

from auditlog.registry import auditlog

auditlog.register(Domain, include_fields=[
    'id', 'domain', 'antivirus', 'antispam', 'userbwlist', 'sendlimit',
    'disabled', 'dkim', 'recvsms', 'sendsms', 'popmail', 'spaceclean',
    'impush', 'share_domains', 'share_title', 'is_wx_host'
])
# auditlog.register(DomainAttr,include_fields=['id','type','item','value'])
auditlog.register(Mailbox, exclude_fields=['last_login', 'domains', 'departments'])
auditlog.register(MailboxUser, exclude_fields=['last_login'])
auditlog.register(Group, include_fields=['id', 'name', 'permissions'])
# auditlog.register(Mailbox.user_permissions.through, exclude_fields=[])
# auditlog.register(Group.permissions.through, exclude_fields=[])
# auditlog.register(MyUserGroups, exclude_fields=[])
# auditlog.register(MyUserPermissions, exclude_fields=[])
# auditlog.register(MyUserPermissions, include_fields=['id', 'user', 'permission'])
# auditlog.register(MyUserGroups, include_fields=['id', 'user', 'group'])

auditlog.register(CoCompany)
auditlog.register(Department, exclude_fields=[])
auditlog.register(CoDepartmentInfo, exclude_fields=[])
auditlog.register(DepartmentMember, exclude_fields=[])
auditlog.register(OabShare, exclude_fields=[])

auditlog.register(CoreTrustIP, exclude_fields=[])
auditlog.register(CoreAlias, exclude_fields=[])
auditlog.register(CoreBlacklist, include_fields=['id', 'email', 'type', 'disabled', 'remark'])
auditlog.register(CoreWhitelist, include_fields=['id', 'email', 'type', 'disabled', 'remark'])
auditlog.register(CoreConfig)
auditlog.register(CoreMonitor)
# auditlog.register(CaptchaStore)

auditlog.register(MailboxExtra, exclude_fields=['size'])
auditlog.register(ExtCommonCheckrule, exclude_fields=[])
auditlog.register(ExtCheckruleCondition, exclude_fields=[])
auditlog.register(ExtReply, exclude_fields=['work_mode', 'work_start', 'work_end'])

auditlog.register(UserDomain, exclude_fields=[])
auditlog.register(UserDepartment, exclude_fields=[])
auditlog.register(UserPhone, exclude_fields=[])
auditlog.register(ExtPasswordReview, include_fields=['username', 'admin', 'new_password', 'status'])
